{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  TechInsite Pty. Ltd.
  PO Box 429, Abbotsford, Melbourne. 3067 Australia
  Phone: +61 3 9419 6456
  Fax:   +61 3 9419 1682
  Web:   www.techinsite.com.au
  EMail: peter_hinrichsen@techinsite.com.au

  Created: 01/06/1999

  Notes: Concrete animal factory. The TFactoryAbstract is descended from with
         the additional function CreateInstanceExt being added. This function
         calls the concrete factory's CreateInstance and adds some type casting.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

unit FactoryConcreteAnimal;

interface
uses
  FactoryAbstract,
  Animals
  ;

type

  // The animal factory
  //----------------------------------------------------------------------------
  TFactoryConcreteAnimal = class( TFactoryAbstract )
  private
  public
    Function CreateInstanceExt( const pStrClassID : string ) :
                                       TAnimal ;

  end ;

// A function with application wide visibility to surfact the unit wide
// variable uFactoryConcreteAnimal
function gFactoryConcreteAnimal : TFactoryConcreteAnimal ;

implementation
var
  // Hold a single instance of our animla factory
  uFactoryConcreteAnimal : TFactoryConcreteAnimal ;

// Make sure there is always an animal factory available
//------------------------------------------------------------------------------
function gFactoryConcreteAnimal : TFactoryConcreteAnimal ;
begin
  if uFactoryConcreteAnimal = nil then
    uFactoryConcreteAnimal := TFactoryConcreteAnimal.Create ;
  result := uFactoryConcreteAnimal ;
end ;

// Create an instance of the required class, but add some type casting
//------------------------------------------------------------------------------
function TFactoryConcreteAnimal.CreateInstanceExt(
            const pStrClassID: string): TAnimal ;
begin
  result := TAnimal( CreateInstance( pStrClassID )) ;
end;

initialization
  gFactoryConcreteAnimal ;

finalization
  uFactoryConcreteAnimal.Free ;

end.
